c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     #############################################################
c     ##                                                         ##
c     ##  subroutine beeman  --  Beeman molecular dynamics step  ##
c     ##                                                         ##
c     #############################################################
c
c
c     "beeman" performs a single molecular dynamics time step
c     via the Beeman multistep recursion formula; uses original
c     coefficients or Bernie Brooks' "Better Beeman" values'
c
c     literature references:
c
c     D. Beeman, "Some Multistep Methods for Use in Molecular
c     Dynamics Calculations", Journal of Computational Physics,
c     20, 130-139 (1976)
c
c     B. R. Brooks, "Algorithms for Molecular Dynamics at Constant
c     Temperature and Pressure", DCRT Report, NIH, April 1988
c
c
      subroutine beeman (istep,dt)
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'freeze.i'
      include 'mdstuf.i'
      include 'moldyn.i'
      include 'molcul.i'
      include 'units.i'
      include 'usage.i'
      include 'deriv.i'
      include 'cross.i'
      include 'boxes.i'
      include 'atmolc.i'
      integer i,j,istep
      real*8 dt,dt_x,factor
c     real*8 tol,over
      real*8 etot,eksum,epot
      real*8 temp,pres
      real*8 part1,part2
      real*8 ekin(3,3)
      real*8 stress(3,3)

      real*8,allocatable:: xold(:),yold(:),zold(:),ders(:,:)
c
c     set time values and coefficients for Beeman integration
c
c     tol=1.0d-4
      factor = dble(bmnmix)
      dt_x = dt / factor
      part1 = 0.5d0*factor + 1.0d0
      part2 = part1 - 2.0d0
c
c     make half-step temperature and pressure corrections
c
      call temper (dt)
c
c     perform dynamic allocation of some local arrays
      allocate(xold(n),yold(n),zold(n),ders(3,n))
c
c     store the current atom positions, then find half-step
c     velocities and full-step positions via Beeman recursion
c
C$OMP PARALLEL DO PRIVATE(i) DEFAULT(SHARED)
      do 1 i = 1, n
      v(1,i) = v(1,i) + (part1*a(1,i)-aalt(1,i))*dt_x
      v(2,i) = v(2,i) + (part1*a(2,i)-aalt(2,i))*dt_x
      v(3,i) = v(3,i) + (part1*a(3,i)-aalt(3,i))*dt_x
      xold(i)=x(i)
      yold(i)=y(i)
      zold(i)=z(i)
      call incr(x(i),v(1,i),dt)
      call incr(y(i),v(2,i),dt)
1     call incr(z(i),v(3,i),dt)

      if(nper.ne.0.and.iamol.ne.0.and.irwp.ne.0.and.rwp2.ne.0.0d0)then
c      damp shifts for rotated molecules:
       call damprot(x,y,z,xold,yold,zold)
      endif
      if(nper.ne.0.and.lstay.and.istep.eq.1)then
c      calculate initial center of mass of the central molecule:   
       call shiftci(x,y,z)
      endif
c
c     get constraint-corrected positions and half-step velocities
c
      if (use_rattle)  call rattle (dt,xold,yold,zold)
c
      twp=0.0d0
      twm=0.0d0

c     get the potential energy and atomic forces:
      call gradient (epot,ders)
      twmi=twmi+twm
      twpi=twpi+twp
c
c     use Newton''s second law to get the next accelerations;
c     find the full-step velocities using the Beeman recursion
c
C$OMP PARALLEL DO PRIVATE(i,j) DEFAULT(SHARED)
      do 3 i = 1, n
       if (use(i)) then
        do 4 j = 1, 3
         aalt(j,i) = a(j,i)
         a(j,i) = -convert * ders(j,i) / mass(i)
4        v(j,i) = v(j,i) + (part2*a(j,i)+aalt(j,i))*dt_x
       end if
3     continue
c
c     shift central molecule to the center
      if(nper.gt.0)call shiftc(x,y,z)
c
c     perform deallocation of some local arrays
c
c     find the constraint-corrected full-step velocities
c
      if (use_rattle)  call rattle2 (dt)
c
c     make full-step temperature and pressure corrections
c
      call temper2 (dt,eksum,ekin,temp)
      call pressure (dt,epot,ekin,temp,pres,stress)
c
c     total energy is sum of kinetic and potential energies
c
      etot = eksum + epot
c
c     compute statistics and save trajectory for this step
c
      call mdstat (istep,dt,etot,epot,eksum,temp,pres)
      call mdsave (istep,dt,epot)
      call mdrest (istep)
      return
      end

      subroutine shiftc(x,y,z)
      implicit none
      include 'sizes.i'
      include 'cross.i'
      include 'molcul.i'
      include 'atmtyp.i'
      integer i,ii,k
      real*8 x(*),y(*),z(*),xce,yce,zce,weight,fa,xt,yt,zt
c
c     calculate mass center of central molecule:
      xce=0.0d0
      yce=0.0d0
      zce=0.0d0
      fa=0.0d0
      do 1 i=1,nper
      k=lper(i)
      weight=mass(k)
      fa=fa+weight
      xce=xce+x(k)*weight
      yce=yce+y(k)*weight
1     zce=zce+z(k)*weight
      xce=xce/fa
      yce=yce/fa
      zce=zce/fa

      if(lstay)then
c      put central molecule to xstay,ystay,zstay
       xt=xstay
       yt=ystay
       zt=zstay
      else
c      put central molecule to 0 0 0
       xt=0.0d0
       yt=0.0d0
       zt=0.0d0
      endif
      do 2 i=1,nper
      ii=lper(i)
      x(ii)=x(ii)-(xce-xt)*0.001d0
      y(ii)=y(ii)-(yce-yt)*0.001d0
2     z(ii)=z(ii)-(zce-zt)*0.001d0

      return
      end

      subroutine shiftci(x,y,z)
      implicit none
      include 'sizes.i'
      include 'cross.i'
      include 'molcul.i'
      include 'atmtyp.i'
      integer i,ii,k
      real*8 x(*),y(*),z(*),weight,fa
c
c     calculate mass center of central molecule:
      xstay=0.0d0
      ystay=0.0d0
      zstay=0.0d0
      fa=0.0d0
      do 1 i=1,nper
      k=lper(i)
      weight=mass(k)
      fa=fa+weight
      xstay=xstay+x(k)*weight
      ystay=ystay+y(k)*weight
1     zstay=zstay+z(k)*weight
      xstay=xstay/fa
      ystay=ystay/fa
      zstay=zstay/fa

      return
      end

      subroutine  incr(x,vx,dt)
      implicit none
      include 'sizes.i'
      include 'cross.i'
      real*8 x,dx,vx,dt
      dx=vx*dt
      if(dx.gt.maxstep0)then
       vx=maxstep0/dt
       dx=maxstep0
      endif
      if(dx.lt.-maxstep0)then
       vx=-maxstep0/dt
       dx=-maxstep0
      endif
      x  = x + dx
      return
      end

      subroutine damprot(x,y,z,xold,yold,zold)
c     damp shifts for rotated molecules
      implicit none
      include 'sizes.i'
      include 'molcul.i'
      include 'cross.i'
      include 'boxes.i'
      include 'atmolc.i'
      integer i,ia,init,iter,ic
      real*8 rm,xmc,ymc,zmc,findda,fi,arg,sf,cf,x0,y0,z0,
     1x0old,y0old,z0old,dx,dy,dz,r,yy,zz,xx,xxold,yyold,zzold,rrot,
     2sc,x(*),y(*),z(*),xold(*),yold(*),zold(*),rwpq,rwp2q,
     3argn,sfn,cfn,tolf
      common/cmc/xmc(maxatm),ymc(maxatm),zmc(maxatm)
      fi=cfi*4.0d0*atan(1.0d0)/180.0d0
      rwpq=rwp**2
      rwp2q=rwp2**2
      tolf=1.1d0

C$OMP  PARALLEL DO PRIVATE(i,rm,init,arg,sf,cf,iter,argn,sfn,cfn,
C$OMP+ ic,ia,x0old,y0old,z0old,x0,y0,z0,dx,dy,dz,r,xxold,yyold,zzold,
C$OMP+ rrot,sc,xx,yy,zz) DEFAULT(SHARED)
      do 100 i=1,nmol
      if(icx.eq.1)rm=ymc(i)**2+zmc(i)**2
      if(icx.eq.2)rm=zmc(i)**2+xmc(i)**2
      if(icx.eq.3)rm=xmc(i)**2+ymc(i)**2
      if(rm.gt.rwpq.and.rm.lt.rwp2q)then
       init=imol(1,i)
       arg=fi*
     1 findda(.false.,iamol,x,y,z,xbox,ybox,zbox,init,moat)
       sf=sin(arg)
       cf=cos(arg)

       iter=0
11     iter=iter+1
c      update mass center
       call calmolceni(i,xmc(i),ymc(i),zmc(i))
c      argument with new mass center
       argn=fi*
     1 findda(.false.,iamol,x,y,z,xbox,ybox,zbox,init,moat)
       sfn=sin(argn)
       cfn=cos(argn)

       ic=0

       if(icx.eq.1)then
        do 1 ia=init,imol(2,i)
        y0old=yold(ia)
        z0old=zold(ia)
        y0=y(ia)
        z0=z(ia)
        iter=iter+1
        dy=y0-y0old
        dz=z0-z0old
        r=dsqrt(dy**2+dz**2)
        call trbb(yyold,zzold,y0old,z0old,cf,sf)
        call trbb(yy,zz,y0,z0,cfn,sfn)
        rrot=dsqrt((yy-yyold)**2+(zz-zzold)**2)
        if(rrot.gt.0.01d0.and.rrot.gt.tolf*r)then
         sc=r/rrot
         y(ia)=y0old+sc*dy
         z(ia)=z0old+sc*dz
         ic=ic+1
        endif
1       continue
       endif

       if(icx.eq.2)then
        do 2 ia=init,imol(2,i)
        z0old=zold(ia)
        x0old=xold(ia)
        z0=z(ia)
        x0=x(ia)
        iter=iter+1
        dz=z0-z0old
        dx=x0-x0old
        r=dsqrt(dz**2+dx**2)
        call trbb(zzold,xxold,z0old,x0old,cf,sf)
        call trbb(zz,xx,z0,x0,cfn,sfn)
        rrot=dsqrt((zz-zzold)**2+(xx-xxold)**2)
        if(rrot.gt.0.01d0.and.rrot.gt.tolf*r)then
         sc=r/rrot
         z(ia)=z0old+sc*dz
         x(ia)=x0old+sc*dx
         ic=ic+1
        endif
2       continue
       endif

       if(icx.eq.3)then
        do 3 ia=init,imol(2,i)
        x0old=xold(ia)
        y0old=yold(ia)
        x0=x(ia)
        y0=y(ia)
        iter=iter+1
        dx=x0-x0old
        dy=y0-y0old
        r=dsqrt(dx**2+dy**2)
        call trbb(xxold,yyold,x0old,y0old,cf,sf)
        call trbb(xx,yy,x0,y0,cfn,sfn)
        rrot=dsqrt((xx-xxold)**2+(yy-yyold)**2)
        if(rrot.gt.0.01d0.and.rrot.gt.tolf*r)then
         sc=r/rrot
         x(ia)=x0old+sc*dx
         y(ia)=y0old+sc*dy
         ic=ic+1
        endif
3       continue
       endif

       if(ic.gt.0.and.iter.lt.2)goto 11

      endif
100   continue

      return
      end
